@@ -1,4 +1,4 @@
-/* @(#)schily.h	1.98 11/08/03 Copyright 1985-2011 J. Schilling */
+/* @(#)schily.h	1.100 11/10/21 Copyright 1985-2011 J. Schilling */
 /*
  *	Definitions for libschily
  *
@@ -214,6 +214,28 @@ extern	int	spawnv_nowait __PR((FILE *, FILE *, FILE *,
 #endif	/* __never_def__ */
 #endif	/* EOF */
 
+#ifdef	FOUND_SIZE_T
+extern	char	*abspath __PR((const char *relp, char *absp, size_t asize));
+#endif
+extern	int	lxchdir	__PR((char *));
+#ifdef	HAVE_FCHDIR
+#define	fdsetname(fd, name)
+#define	fdclosename(fd)
+#else
+extern	int	fdsetname __PR((int fd, const char *name));
+extern	int	fdclosename __PR((int fd));
+#endif
+
+struct save_wd {
+	int	fd;
+	char	*name;
+};
+
+extern int	savewd	__PR((struct save_wd *sp));
+extern void	closewd	__PR((struct save_wd *sp));
+extern int	restorewd __PR((struct save_wd *sp));
+
+
 #ifdef	_SCHILY_UTYPES_H
 typedef struct gnmult {
 	char	key;
@@ -410,6 +432,7 @@ extern	int	qftofs __PR((char *, long double, int, int));
 /*PRINTFLIKE2*/
 extern	int	js_fprintf	__PR((FILE *, const char *, ...))
 							__printflike__(2, 3);
+#endif	/* EOF */
 /*PRINTFLIKE1*/
 extern	int	js_printf	__PR((const char *, ...)) __printflike__(1, 2);
 #ifdef	FOUND_SIZE_T
@@ -420,7 +443,6 @@ extern	int	js_snprintf	__PR((char *, size_t, const char *, ...))
 /*PRINTFLIKE2*/
 extern	int	js_sprintf	__PR((char *, const char *, ...))
 							__printflike__(2, 3);
-#endif	/* EOF */
 
 #ifdef	FOUND_SIZE_T
 extern	void	swabbytes	__PR((void *, ssize_t));
