@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2011 J. Schilling
  *
- * @(#)admin.c	1.66 11/09/15 J. Schilling
+ * @(#)admin.c	1.73 11/11/01 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)admin.c 1.66 11/09/15 J. Schilling"
+#pragma ident "@(#)admin.c 1.73 11/11/01 J. Schilling"
 #endif
 /*
  * @(#)admin.c 1.39 06/12/12
@@ -100,18 +100,6 @@ should appear exactly as they do in the msgid string:
 # define COPY 0
 # define NOCOPY 1
 
-struct stat Statbuf;
-
-char	Null[1];
-char	SccsError[MAXERRORLEN];
-char	*Comments, *Mrs;
-
-int	Did_id;
-int	Domrs;
-
-extern FILE *Xiop;
-extern char *Sflags[];
-
 static char	stdin_file_buf [20];
 static char	*ifile, *tfile;
 static char	*dir_name;
@@ -148,7 +136,7 @@ static off_t	Checksum_offset;	/* offset in file where g-file hash is stored */
 static	void	admin __PR((char *afile));
 static	int	fgetchk __PR((FILE *inptr, char *file, struct packet *pkt, int fflag));
 static	void	warnctl __PR((char *file, off_t nline));
-	void	clean_up __PR((void));
+static	void	clean_up __PR((void));
 static	void	cmt_ba __PR((register struct deltab *dt, char *str, int flags));
 static	void	putmrs __PR((struct packet *pkt));
 static	char *	adjust __PR((char *line));
@@ -164,8 +152,6 @@ extern int	org_ihash;
 extern int	org_chash;
 extern int	org_uchash;
 
-extern char	saveid[];
-
 int
 main(argc,argv)
 int argc;
@@ -200,10 +186,14 @@ char *argv[];
 
 	tzset();	/* Set up timezome related vars */
 
+#ifdef	SCHILY_BUILD
+	save_args(argc, argv);
+#endif
 	/*
 	Set flags for 'fatal' to issue message, call clean-up
 	routine and terminate processing.
 	*/
+	set_clean_up(clean_up);
 	Fflags = FTLMSG | FTLCLN | FTLEXIT;
 
 	testklt = 1;
@@ -528,9 +518,10 @@ char *argv[];
 					exit(EX_OK);
 				}
 				if (p[1] == '\0') {
-					if (p[0] == '4')
+					if (p[0] == '4') {
+						versflag = 4;
 						break;
-					if (p[0] == '6') {
+					} else if (p[0] == '6') {
 						versflag = 6;
 						break;
 					}
@@ -603,6 +594,11 @@ char *argv[];
 
 	setsig();
 
+	errno = 0;
+	i = sethome();
+	if (i < 0)
+		efatal(gettext("cannot get project home directory (cm11)"));
+
 	/*
 	Change flags for 'fatal' so that it will return to this
 	routine (main) instead of terminating processing.
@@ -653,7 +649,6 @@ char	*afile;
 	char	*lval;
 	char	f;		/* character holder for flag character */
 	char	line[BUFSIZ];
-	int	i;		/* used in forking procedure */
 	int	ck_it;		/* used for lockflag checking */
 	off_t	offset;
 	int     thash;
@@ -720,13 +715,15 @@ char	*afile;
 		fatal(SccsError);
 	}
 	if (HADH) {
+		pid_t	pid;
+
 		/*
 		   fork here so 'admin' can execute 'val' to
 		   check for a corrupted file.
 		*/
-		if ((i = vfork()) < 0)
+		if ((pid = vfork()) < 0)
 			fatal(gettext("cannot fork, try again"));
-		if (i == 0) {		/* child */
+		if (pid == 0) {		/* child */
 			/*
 			   perform 'val' with appropriate keyletters
 			*/
@@ -809,6 +806,27 @@ char	*afile;
 		}
 
 		sinit(&gpkt,afile,0);	/* and init pkt */
+
+		/*
+		 * Initialize global meta data
+		 */
+		if (versflag == 6 && setrhome != NULL &&
+		    afile[0] != '/' &&
+		    !(afile[0] == '.' && afile[1] == '.' && afile[2] == '/') &&
+		    !strstr(afile, "/../")) {
+			char	*p;
+
+			sprintf(line, "%s%s%s%s%s",
+				homedist > 0 ? cwdprefix:"",
+				homedist > 0 ? "/": "",
+				dir_name, *dir_name ? "/":"",
+				auxf(afile, 'g'));
+			p = fmalloc(size(line));
+			strcpy(p, line);
+			gpkt.p_init_path = p;
+		}
+		if (versflag == 6)
+			urandom(&gpkt.p_rand);
 	}
 
 	if (!HADH)
@@ -904,7 +922,7 @@ char	*afile;
 			putmrs(&gpkt);
 		}
 		if (gpkt.p_flags & PF_V6) {
-			Checksum_offset = ftell(Xiop);
+			Checksum_offset = ftell(gpkt.p_xiop);
 			sidext_ba(&gpkt, &dt);	/* Will not write "dt" entry. */
 		}
 
@@ -1081,6 +1099,8 @@ char	*afile;
 	}
 	for (k = 0; k < NFLAGS; k++)
 		if (had_flag[k]) {
+			int	i;		/* for flag string cleanup */
+
 			if (flag_p[k] || lval ) {
 				if (('a' + k) == LOCKFLAG && had_flag[k] == 1) {
 					if ((flag_p[k] && *flag_p[k] == 'a') || (lval && *lval == 'a'))
@@ -1138,11 +1158,13 @@ char	*afile;
 			non-ASCII characters we can flag it as encoded
 			by setting the value to 1.
 		 	*/
-			Encodeflag_offset = ftell(Xiop);
+			Encodeflag_offset = ftell(gpkt.p_xiop);
 			sprintf(line,"%c%c %c %d\n",
 				CTLCHAR, FLAG, ENCODEFLAG, Encoded);
 			putline(&gpkt,line);
 		}
+		putmeta(&gpkt);
+
 		/*
 		Beginning of descriptive (user) text.
 		*/
@@ -1210,7 +1232,7 @@ char	*afile;
 			Set indicator that tells whether there
 			were any keywords to 'no'.
 			*/
-			Did_id = 0;
+			gpkt.p_did_id = 0;
 			if (ifile) {
 			   if (*ifile) {
 				/* from a file */
@@ -1269,7 +1291,7 @@ char	*afile;
                            file.  Then won't have to start all over.  Also
                            save the hash value up to this point.
 			 */
-			offset = ftell(Xiop);
+			offset = ftell(gpkt.p_xiop);
 			thash = gpkt.p_nhash;
 
 			/*
@@ -1309,7 +1331,7 @@ char	*afile;
 			 * NSE mode...or warnings have been disabled via an
 			 * empty 'y' flag value.
 			 */
-			if (!Did_id && !HADQ &&
+			if (!gpkt.p_did_id && !HADQ &&
 			    (!flag_p[EXPANDFLAG - 'a'] ||
 			    *(flag_p[EXPANDFLAG - 'a']))) {
 				if (had_flag[IDFLAG - 'a']) {
@@ -1323,7 +1345,7 @@ char	*afile;
 			}
 
 			check_id = 0;
-			Did_id = 0;
+			gpkt.p_did_id = 0;
 		}
 
 		/*
@@ -1332,8 +1354,8 @@ char	*afile;
 		sprintf(line,"%c%c %d\n",CTLCHAR,END,1);
 		putline(&gpkt,line);
 		if (gpkt.p_flags & PF_V6 && gpkt.p_ghash != 0) {
-			fseek(Xiop, Checksum_offset, SEEK_SET);
-			fprintf(Xiop, "%c%c s %5.5d\n",
+			fseek(gpkt.p_xiop, Checksum_offset, SEEK_SET);
+			fprintf(gpkt.p_xiop, "%c%c s %5.5d\n",
 				CTLCHAR, SIDEXTENS, gpkt.p_ghash);
 			gpkt.p_nhash -= 5 * '0';
 			sprintf(line, "%5.5d", gpkt.p_ghash);
@@ -1364,8 +1386,8 @@ char	*afile;
 		q = (signed char *) line;
 		while (*q)
 			gpkt.p_nhash += *q++;
-		fseek(Xiop, Encodeflag_offset, SEEK_SET);
-		fprintf(Xiop,"%c%c %c %d\n",
+		fseek(gpkt.p_xiop, Encodeflag_offset, SEEK_SET);
+		fprintf(gpkt.p_xiop,"%c%c %c %d\n",
 			CTLCHAR, FLAG, ENCODEFLAG, Encoded);
 	}
 
@@ -1400,7 +1422,7 @@ char	*afile;
 			if (strlcat(cfile, snp, sizeof (cfile)) < sizeof (cfile))
 				rename(ifile, cfile);
 		}
-		xrm();
+		xrm(&gpkt);
 		uname(&un);
 		uuname = un.nodename;
 		unlockit(auxf(afile,'z'),getpid(),uuname);
@@ -1408,8 +1430,14 @@ char	*afile;
 
 	if (HADI)
 		unlink(auxf(gpkt.p_file,'e'));
-	if (from_stdin)
+	if (from_stdin) {
 		unlink(stdin_file_buf);
+		stdin_file_buf[0] = '\0';
+	}
+	if (gpkt.p_init_path) {
+		ffree(gpkt.p_init_path);
+		gpkt.p_init_path = NULL;
+	}
 }
 
 static int
@@ -1435,7 +1463,7 @@ struct	packet	*pkt;		/* struct paket for output	*/
 #endif
 	off_t	ibase = 0;	/* Ifile off from last read operation	*/
 	off_t	ioff = 0;	/* Ifile offset past last newline	*/
-	off_t	soff = ftell(Xiop); /* Ofile (s. file) base offset	*/
+	off_t	soff = ftell(pkt->p_xiop); /* Ofile (s. file) base offset */
 	unsigned int sum = 0;
 
 	/*
@@ -1514,8 +1542,9 @@ struct	packet	*pkt;		/* struct paket for output	*/
 		line[idx] = '\0';
 		putline(pkt, lastline);
 
-		if (check_id && Did_id == 0) {
-			(void)chkid(line, flag_p[IDFLAG - 'a'], flag_p);
+		if (check_id && pkt->p_did_id == 0) {
+			pkt->p_did_id =
+				chkid(line, flag_p[IDFLAG - 'a'], flag_p);
 		}
 		ibase += idx;
 	}
@@ -1563,8 +1592,9 @@ struct	packet	*pkt;		/* struct paket for output	*/
 		 }
 	      }
 	   }   
-	   if (check_id && Did_id == 0) {
-	      (void)chkid(line, flag_p[IDFLAG - 'a'], flag_p);
+	   if (check_id && pkt->p_did_id == 0) {
+		pkt->p_did_id =
+			chkid(line, flag_p[IDFLAG - 'a'], flag_p);
 	   }
 	   putline(pkt, line);
 	   (void)memset(line, '\377', sizeof (line));
@@ -1582,12 +1612,12 @@ struct	packet	*pkt;		/* struct paket for output	*/
 #endif
 		if (fflag && (pkt->p_flags & PF_V6)) {
 			fseek(inptr, ioff, SEEK_SET);
-			fseek(Xiop, ioff + soff, SEEK_SET);
+			fseek(pkt->p_xiop, ioff + soff, SEEK_SET);
 
 			putctlnnl(pkt);
 			while ((idx =
 			    fread(line, 1, sizeof (line) - 1, inptr)) > 0) {
-				if (fwrite(line, 1, idx, Xiop) <= 0)
+				if (fwrite(line, 1, idx, pkt->p_xiop) <= 0)
 					FAILPUT;
 			}
 			putchr(pkt, '\n');
@@ -1624,13 +1654,13 @@ warnctl(file, nline)
 		file, (intmax_t)nline);
 }
  
-void
+static void
 clean_up()
 {
-	xrm();
-	if (Xiop) {
-		fclose(Xiop);
-		Xiop = NULL;
+	xrm(&gpkt);
+	if (gpkt.p_xiop) {
+		fclose(gpkt.p_xiop);
+		gpkt.p_xiop = NULL;
 	}
 	if(gpkt.p_file[0]) {
 		unlink(auxf(gpkt.p_file,'x'));
@@ -1639,6 +1669,10 @@ clean_up()
 		if (HADN)
 			unlink(gpkt.p_file);
 	}
+	if (gpkt.p_init_path) {
+		ffree(gpkt.p_init_path);
+		gpkt.p_init_path = NULL;
+	}
 	if (!HADH) {
 		uname(&un);
 		uuname = un.nodename;
@@ -1847,7 +1881,7 @@ struct packet *pktp;
 	 * the end of sccs header info and before gfile contents
 	 */
 	putline(pktp,0);
-	fseek(Xiop, offset, SEEK_SET);
+	fseek(pktp->p_xiop, offset, SEEK_SET);
 	pktp->p_nhash = thash;
 
 	return (iptr);
@@ -1882,7 +1916,7 @@ static char	Nhold[MAXPATHLEN];
 
 #ifdef	HAVE_FCHDIR
 	if (dfd < 0) {
-		dfd = open(".", 0);		/* on failure use full path */
+		dfd = open(".", O_SEARCH);	/* on failure use full path */
 	} else {
 		if (fchdir(dfd) < 0)
 			xmsg(".",NOGETTEXT("admin"));
@@ -1906,11 +1940,26 @@ static char	Nhold[MAXPATHLEN];
 			Dir[0] = '\0';
 		} else if (*afile == '/' && &afile[1] == sn) {
 			copy("/", Dir);
-		} else {
+		} else {			/* Get dir part from afile */
 			size_t	len = sn - afile;
 			if (len > sizeof (Dir))
 				len = sizeof (Dir);
 			strlcpy(Dir, afile, len); /* replace last '/' by '\0' */
+			/*
+			 * fixpath() (formerly abspath() from AT&T) is buggy: it
+			 * is unable to remove "./" at the beginning and it
+			 * returns -1 leaving a corrupted path buffer in case
+			 * it contained too many "/../" entries.
+			 * resolvepath() is available as syscall on Solaris,
+			 * currently missing elsewhere. We however need a path
+			 * free of symlink components.
+			 */
+#ifdef	HAVE_RESOLVEPATH
+			if (resolvepath(Dir, Dir, sizeof (Dir)) == -1)
+#else
+			if (fixpath(Dir) == (char *)-1)
+#endif
+				fatal(gettext("path conversion error (cm12)"));
 		}
 		if (Dir[0] != '\0' && (dfd < 0 || chdir(Dir) < 0)) {
 			cat(Nhold, Dir, *Dir?"/":"", Nprefix, sn, (char *)0);
@@ -1919,8 +1968,11 @@ static char	Nhold[MAXPATHLEN];
 			cat(Nhold, Nprefix, sn, (char *)0); /* use short name */
 			ifile = sn;
 			dir_name = Dir;
+			if (dir_name[0] == '.' && dir_name[1] == '/')
+				dir_name += 2;
 		}
 		afile = Nhold;			/* Use computed s.file name */
+
 	} else {				/* afile is s.file name */
 		char	*np;
 		size_t	plen = strlen(Nprefix);
@@ -1942,9 +1994,17 @@ static char	Nhold[MAXPATHLEN];
 					copy("/", Dir);
 				else
 					copy(afile, Dir);
+#ifdef	HAVE_RESOLVEPATH
+				if (resolvepath(Dir, Dir, sizeof (Dir)) == -1)
+#else
+				if (fixpath(Dir) == (char *)-1)
+#endif
+					fatal(gettext("path conversion error (cm12)"));
 				if (chdir(Dir) < 0)
 					fatal("Chdir");
 				dir_name = Dir;
+				if (dir_name[0] == '.' && dir_name[1] == '/')
+					dir_name += 2;
 				afile = np;
 				copy(auxf(np,'g'), Nhold);
 			} else {
