@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2011 J. Schilling
  *
- * @(#)help2.c	1.11 11/09/30 J. Schilling
+ * @(#)help2.c	1.14 11/10/17 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)help2.c 1.11 11/09/30 J. Schilling"
+#pragma ident "@(#)help2.c 1.14 11/10/17 J. Schilling"
 #endif
 /*
  * @(#)help2.c 1.10 06/12/12
@@ -113,9 +113,7 @@ static char helpdir[] = NOGETTEXT(INS_BASE "/ccs/lib/help/locale/");
 static char helpdir[] = NOGETTEXT("/usr/ccs/lib/help/locale/");
 #endif
 
-char	SccsError[MAXERRORLEN];
 static char	hfile[256];
-struct	stat	Statbuf;
 static FILE	*iop;
 static char	line [MAXLINE+1];
 static char   *locale = NULL; /* User's locale. */
@@ -195,8 +193,7 @@ main(argc, argv)
 	if (argc == 1) {
 		char	*he = ask();
 		if (*he == '\0') {
-			numerrs += findprt("sccs");
-			numerrs += findprt("sccs_subcmds");
+			numerrs += findprt("intro");
 		} else {
 			numerrs += findprt(he);
 		}
@@ -343,9 +340,3 @@ lochelp(ky, fi, fisize)
 	fclose(fp);
 	return(0); /* no entry found */
 }
-
-/* for fatal() */
-void
-clean_up()
-{
-}
