@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2011 J. Schilling
  *
- * @(#)prs.c	1.33 11/09/15 J. Schilling
+ * @(#)prs.c	1.39 11/10/21 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)prs.c 1.33 11/09/15 J. Schilling"
+#pragma ident "@(#)prs.c 1.39 11/10/21 J. Schilling"
 #endif
 /*
  * @(#)prs.c 1.33 06/12/12
@@ -75,10 +75,6 @@
 # include	<schily/vfork.h>
 # include	<schily/sysexits.h>
 
-struct stat Statbuf;
-char Null[1];
-char SccsError[MAXERRORLEN];
-
 #if	defined(PROTOTYPES) && defined(INS_BASE)
 static char	Getpgmp[]   =   NOGETTEXT(INS_BASE "/ccs/bin/" "get");
 #endif
@@ -97,7 +93,7 @@ static char	*Deltatime;
 static char	tempskel[]   =   NOGETTEXT("/tmp/prXXXXXX");	/* used to generate temp
 						   file names
 						*/
-struct	sid	sid;
+static struct	sid	sid;
 
 static char	untmp[32], uttmp[32], cmtmp[32];
 static char	mrtmp[32], sxtmp[32], bdtmp[32];
@@ -118,7 +114,7 @@ static struct	tm	*Dtime;
 static struct tm 	Date_time;
 static struct tm 	Del_Date_time;
 
-	void    clean_up __PR((void));
+static void	clean_up __PR((void));
 
 	int	main __PR((int argc, char **argv));
 static void	process __PR((char *file));
@@ -183,6 +179,7 @@ char *argv[];
 	Set flags for 'fatal' to issue message, call clean-up
 	routine, and terminate processing.
 	*/
+	set_clean_up(clean_up);
 	Fflags = FTLMSG | FTLCLN | FTLEXIT;
 
 	/*
@@ -387,7 +384,7 @@ register	char	*file;
 	else read_to(EUSERNAM,&gpkt);
 
 	/*
-	store flags (if any) into global array called 'Sflags'
+	store flags (if any) into array called 'gpkt.p_sflags'
 	*/
 
 	doflags(&gpkt);
@@ -603,7 +600,6 @@ register struct packet *pkt;
  * immediately.
 */
 
-extern	char	*Sflags[];
 static	char	Zkywd[5]   =   "@(#)";
 
 static void
@@ -678,7 +674,7 @@ struct	stats	*statp;
 				printf("%s",Qsect);
 				break;
 			case 'J':	/* joint edit flag */
-				if (Sflags[JOINTFLAG - 'a'])
+				if (gpkt.p_sflags[JOINTFLAG - 'a'])
 					printf(gettext("yes"));
 				else printf(gettext("no"));
 				break;
@@ -774,11 +770,23 @@ struct	stats	*statp;
 				printf("%d",dtp->d_pred);
 				break;
 			case 256*'I'+'D':	/* :DI: Deltas included,excluded,ignored */
+#ifdef	ATT_BUG
+				/*
+				 * This has been introduced around 1984.
+				 */
 				printf("%s",iline);
 				if (length(xline))
 					printf("/%s",xline);
 				if (length(gline))
 					printf("/%s",gline);
+#else
+				/*
+				 * This is the POSIX compliant behavior.
+				 */
+				printf("%s",iline);
+				printf("/%s",xline);
+				printf("/%s",gline);
+#endif
 				break;
 			case 256*'n'+'D':	/* :Dn: Deltas included */
 				printf("%s",iline);
@@ -790,7 +798,7 @@ struct	stats	*statp;
 				printf("%s",gline);
 				break;
 			case 256*'K'+'L':	/* :LK: locked releases */
-				if ((k = Sflags[LOCKFLAG - 'a']) != NULL)
+				if ((k = gpkt.p_sflags[LOCKFLAG - 'a']) != NULL)
 					printf("%s",k);
 				else printf(gettext("none"));
 				break;
@@ -799,7 +807,7 @@ struct	stats	*statp;
 					printfile(mrtmp);
 				break;
 			case 256*'C'+'M':  /* :MC: cmf flag yes or no */
-				if(Sflags[CMFFLAG - 'a'])
+				if (gpkt.p_sflags[CMFFLAG - 'a'])
 					printf(gettext("yes"));
 				else
 					printf(gettext("no"));
@@ -809,7 +817,7 @@ struct	stats	*statp;
 					printfile(sxtmp);
 				break;
 			case 256*'C'+'A':  /* :AC: cmf application field */
-				if((k = Sflags[CMFFLAG - 'a']) == NULL)
+				if ((k = gpkt.p_sflags[CMFFLAG - 'a']) == NULL)
 					printf(gettext("none"));
 				else
 					printf("%s",k);
@@ -819,42 +827,42 @@ struct	stats	*statp;
 					printfile(untmp);
 				break;
 			case 256*'F'+'M':	/* :MF: MR validation flag */
-				if (Sflags[VALFLAG - 'a'])
+				if (gpkt.p_sflags[VALFLAG - 'a'])
 					printf(gettext("yes"));
 				else printf(gettext("no"));
 				break;
 			case 256*'P'+'M':	/* :MP: MR validation program */
-				if ((k = Sflags[VALFLAG - 'a']) == NULL)
+				if ((k = gpkt.p_sflags[VALFLAG - 'a']) == NULL)
 					printf(gettext("none"));
 				else printf("%s",k);
 				break;
 			case 256*'F'+'K':	/* :KF: Keyword err/warn flag */
-				if (Sflags[IDFLAG - 'a'])
+				if (gpkt.p_sflags[IDFLAG - 'a'])
 					printf(gettext("yes"));
 				else printf(gettext("no"));
 				break;
 			case 256*'F'+'B':	/* :BF: Branch flag */
-				if (Sflags[BRCHFLAG - 'a'])
+				if (gpkt.p_sflags[BRCHFLAG - 'a'])
 					printf(gettext("yes"));
 				else printf(gettext("no"));
 				break;
 			case 256*'B'+'F':	/* :FB: Floor Boundry */
-				if ((k = Sflags[FLORFLAG - 'a']) != NULL)
+				if ((k =gpkt.p_sflags[FLORFLAG - 'a']) != NULL)
 					printf("%s",k);
 				else printf(gettext("none"));
 				break;
 			case 256*'B'+'C':	/* :CB: Ceiling Boundry */
-				if ((k = Sflags[CEILFLAG - 'a']) != NULL)
+				if ((k = gpkt.p_sflags[CEILFLAG - 'a']) != NULL)
 					printf("%s",k);
 				else printf(gettext("none"));
 				break;
 			case 256*'s'+'D':	/* :Ds: Default SID */
-				if ((k = Sflags[DEFTFLAG - 'a']) != NULL)
+				if ((k = gpkt.p_sflags[DEFTFLAG - 'a']) != NULL)
 					printf("%s",k);
 				else printf(gettext("none"));
 				break;
 			case 256*'D'+'N':	/* :ND: Null delta */
-				if (Sflags[NULLFLAG - 'a'])
+				if (gpkt.p_sflags[NULLFLAG - 'a'])
 					printf(gettext("yes"));
 				else printf(gettext("no"));
 				break;
@@ -958,14 +966,14 @@ struct	stats	*statp;
  * 'process' that are used for data keyword substitution
 */
 
-void
+static void
 clean_up()
 {
 	if (gpkt.p_iop) {	/* if SCCS file is open, close it */
 		fclose(gpkt.p_iop);
 		gpkt.p_iop = NULL;
 	}
-	xrm();		      /* remove the 'packet' used for this SCCS file */
+	xrm(&gpkt);	      /* remove the 'packet' used for this SCCS file */
 	unlink(mrtmp);		/* remove all temporary files from /tmp */
 	unlink(cmtmp);		/*			"		*/
 	unlink(sxtmp);		/*			"		*/
@@ -1430,12 +1438,12 @@ time_t	*bdate;
 #if !(defined(BUG_1205145) || defined(GMT_TIME))
 	Dtime = localtime(bdate);
 #endif
-	if ((p = Sflags[MODFLAG - 'a']) != NULL)
+	if ((p = pkt->p_sflags[MODFLAG - 'a']) != NULL)
 		copy(p,Mod);
 	else sprintf(Mod,"%s",auxf(pkt->p_file,'g'));
-	if ((Type = Sflags[TYPEFLAG - 'a']) == NULL)
+	if ((Type = pkt->p_sflags[TYPEFLAG - 'a']) == NULL)
 		Type = Null;
-	if ((Qsect = Sflags[QSECTFLAG - 'a']) == NULL)
+	if ((Qsect = pkt->p_sflags[QSECTFLAG - 'a']) == NULL)
 		Qsect = Null;
 }
 
@@ -1537,14 +1545,15 @@ static void
 printflags()
 {
 	register	char	*k;
+	register	char	**sflags = gpkt.p_sflags;
 
-	if (Sflags[BRCHFLAG - 'a'])	/* check for 'branch' flag */
+	if (sflags[BRCHFLAG - 'a'])			/* check for 'branch' flag */
 		printf(gettext("branch\n"));
-	if ((k = (Sflags[CEILFLAG - 'a'])) != NULL)	/* check for 'ceiling flag */
+	if ((k = (sflags[CEILFLAG - 'a'])) != NULL)	/* check for 'ceiling flag */
 		printf(gettext("ceiling\t%s\n"), k);
-	if ((k = (Sflags[DEFTFLAG - 'a'])) != NULL)  /* check for 'default SID' flag */
+	if ((k = (sflags[DEFTFLAG - 'a'])) != NULL)	/* check for 'default SID' flag */
 		printf(gettext("default SID\t%s\n"), k);
-	if ((k = (Sflags[ENCODEFLAG - 'a'])) != NULL) {	/* check for 'encoded' flag */
+	if ((k = (sflags[ENCODEFLAG - 'a'])) != NULL) {	/* check for 'encoded' flag */
 		int	i;
 
 		NONBLANK(k);
@@ -1557,37 +1566,37 @@ printflags()
 		if (*k == '\0' && (i & EF_UUENCODE))
 			printf(gettext("encoded\n"));
 	}
-	if ((k = (Sflags[FLORFLAG - 'a'])) != NULL)	/* check for 'floor' flag */
+	if ((k = (sflags[FLORFLAG - 'a'])) != NULL)	/* check for 'floor' flag */
 		printf(gettext("floor\t%s\n"), k);
-	if (Sflags[IDFLAG - 'a'])	/* check for 'id err/warn' flag */
+	if (sflags[IDFLAG - 'a'])			/* check for 'id err/warn' flag */
 		printf(gettext("id keywd err/warn\n"));
-	if (Sflags[JOINTFLAG - 'a'])	/* check for joint edit flag */
+	if (sflags[JOINTFLAG - 'a'])			/* check for joint edit flag */
 		printf(gettext("joint edit\n"));
-	if ((k = (Sflags[LOCKFLAG - 'a'])) != NULL)	/* check for 'lock' flag */
+	if ((k = (sflags[LOCKFLAG - 'a'])) != NULL)	/* check for 'lock' flag */
 		printf(gettext("locked releases\t%s\n"), k);
-	if ((k = (Sflags[MODFLAG - 'a'])) != NULL)	/* check for 'module' flag */
+	if ((k = (sflags[MODFLAG - 'a'])) != NULL)	/* check for 'module' flag */
 		printf(gettext("module\t%s\n"), k);
-	if (Sflags[NULLFLAG - 'a'])	/* check for 'null delta' flag */
+	if (sflags[NULLFLAG - 'a'])			/* check for 'null delta' flag */
 		printf(gettext("null delta\n"));
-	if ((k = (Sflags[QSECTFLAG - 'a'])) != NULL)	/* check for 'qsect' flag */
+	if ((k = (sflags[QSECTFLAG - 'a'])) != NULL)	/* check for 'qsect' flag */
 		printf(gettext("csect name\t%s\n"), k);
-	if ((k = (Sflags[TYPEFLAG - 'a'])) != NULL)	/* check for 'type' flag */
+	if ((k = (sflags[TYPEFLAG - 'a'])) != NULL)	/* check for 'type' flag */
 		printf(gettext("type\t%s\n"), k);
-	if (Sflags[VALFLAG - 'a']) {	/* check for 'MR valid' flag */
+	if (sflags[VALFLAG - 'a']) {			/* check for 'MR valid' flag */
 		printf(gettext("validate MRs\t"));
 		/*
 		check for MR validating program
 		(optional)
 		*/
-		if ((k = (Sflags[VALFLAG - 'a'])) != NULL)
+		if ((k = (sflags[VALFLAG - 'a'])) != NULL)
 			printf("%s\n",k);
 		else putchar('\n');
 	}
-	if ((k = (Sflags[SCANFLAG - 'a'])) != NULL)	/* check for 'keywd scan lines' flag */
+	if ((k = (sflags[SCANFLAG - 'a'])) != NULL)	/* check for 'keywd scan lines' flag */
 		printf(gettext("keywd scan lines\t%s\n"), k);
-	if ((k = (Sflags[EXTENSFLAG - 'a'])) != NULL)	/* check for 'extensions' flag */
+	if ((k = (sflags[EXTENSFLAG - 'a'])) != NULL)	/* check for 'extensions' flag */
 		printf(gettext("extensions\t%s\n"), k);
-	if ((k = (Sflags[EXPANDFLAG - 'a'])) != NULL)	/* check for 'expand keywds' flag */
+	if ((k = (sflags[EXPANDFLAG - 'a'])) != NULL)	/* check for 'expand keywds' flag */
 		printf(gettext("expand keywds\t%s\n"), k);
 	return;
 }
