@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2011 J. Schilling
  *
- * @(#)prt.c	1.27 11/09/15 J. Schilling
+ * @(#)prt.c	1.29 11/10/08 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)prt.c 1.27 11/09/15 J. Schilling"
+#pragma ident "@(#)prt.c 1.29 11/10/08 J. Schilling"
 #endif
 /*
  * @(#)prt.c 1.22 06/12/12
@@ -69,8 +69,6 @@
 # define NOEOF	0
 # define BLANK(p)	while (!(*p == '\0' || *p == ' ' || *p == '\t')) p++;
 
-char SccsError[MAXERRORLEN];
-struct stat Statbuf;
 static FILE *iptr;
 static char *line = NULL;
 static size_t line_size = 0;
@@ -691,9 +689,3 @@ register char *str, *cp;
 
 	printf("%s%s",str,cp);
 }
-
-/* for fatal() */
-void
-clean_up()
-{
-}
