@@ -1,8 +1,8 @@
-/* @(#)sccscvt.c	1.6 11/09/28 Copyright 2011 J. Schilling */
+/* @(#)sccscvt.c	1.12 11/10/13 Copyright 2011 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)sccscvt.c	1.6 11/09/28 Copyright 2011 J. Schilling";
+	"@(#)sccscvt.c	1.12 11/10/13 Copyright 2011 J. Schilling";
 #endif
 /*
  *	Convert a SCCS v4 history file to a SCCS v6 file and vice versa.
@@ -41,12 +41,7 @@ LOCAL	void	cvtdelt2v4	__PR((struct packet *pkt));
 LOCAL	void	cvtdelt2v6	__PR((struct packet *pkt));
 LOCAL	void	get_setup	__PR((char *file));
 LOCAL	int	get_hash	__PR((int ser));
-EXPORT	void    clean_up	__PR((void));
-EXPORT	void	escdodelt	__PR((struct packet *pkt));
-EXPORT	void	fredck		__PR((struct packet *pkt));
-
-EXPORT	struct stat	Statbuf;
-EXPORT	char		SccsError[MAXERRORLEN];
+LOCAL	void	clean_up	__PR((void));
 
 LOCAL	struct utsname	un;
 LOCAL	char		*uuname;
@@ -56,8 +51,6 @@ LOCAL	BOOL	dov6	= -1;
 LOCAL	BOOL	keepold;
 LOCAL	BOOL	discardv6;
 
-extern FILE	*Xiop;
-
 LOCAL void
 usage(exitcode)
 	int	exitcode;
@@ -106,6 +99,7 @@ main(ac, av)
 
 	tzset();	/* Set up timezome related vars */
 
+	set_clean_up(clean_up);
 	Fflags = FTLEXIT | FTLMSG | FTLCLN;
 
 	cac = --ac;
@@ -203,8 +197,28 @@ convert(file)
 	if (setjmp(Fjmp))
 		return;
 
+	if (!sccsfile(file)) {
+		errmsgno(EX_BAD, _("%s: not an SCCS file (co1).\n"), file);
+		return;
+	}
+
 	/*
-	 * Open s. file and check whether it is a SCCS file.
+	 * Init and check for validity of file name but do not open the file.
+	 * This prevents us from potentially damaging files with lockit().
+	 */
+	sinit(&gpkt, file, 0);
+
+	/*
+	 * Obtain a lock on the SCCS history file.
+	 */
+	uname(&un);
+	uuname = un.nodename;
+	if (lockit(auxf(gpkt.p_file, 'z'),
+	    SCCS_LOCK_ATTEMPTS, getpid(), uuname))
+		fatal(_("cannot create lock file (cm4)"));
+
+	/*
+	 * Open s. file.
 	 */
 	sinit(&gpkt, file, 1);
 	if (dov6) {
@@ -231,15 +245,6 @@ convert(file)
 		gpkt.p_flags &= ~PF_V6;	/* Make sure initial chksum is V4 */
 	}
 
-	/*
-	 * As we found a SCCS history file, we may now obtain a lock.
-	 */
-	uname(&un);
-	uuname = un.nodename;
-	if (lockit(auxf(gpkt.p_file, 'z'),
-	    SCCS_LOCK_ATTEMPTS, getpid(), uuname))
-		fatal(_("cannot create lock file (cm4)"));
-
 	/*
 	 * Write a magic in the expected new format.
 	 */
@@ -285,7 +290,7 @@ convert(file)
 	 * Make sure the data is stable in the file on disk.
 	 */
 	xf = auxf(gpkt.p_file, 'x');
-	if (fflush(Xiop) == EOF)
+	if (fflush(gpkt.p_xiop) == EOF)
 		xmsg(xf, NOGETTEXT("convert"));
 
 	/*
@@ -293,12 +298,12 @@ convert(file)
 	 * delayed failure information from NFS.
 	 */
 #ifdef	HAVE_FSYNC
-	if (fsync(fileno(Xiop)) < 0)
+	if (fsync(fileno(gpkt.p_xiop)) < 0)
 		xmsg(xf, NOGETTEXT("convert"));
 #endif
-	if (fclose(Xiop) == EOF)
+	if (fclose(gpkt.p_xiop) == EOF)
 		xmsg(xf, NOGETTEXT("flushline"));
-	Xiop = NULL;
+	gpkt.p_xiop = NULL;
 
 	stat(gpkt.p_file, &sbuf);
 	if (keepold)
@@ -625,7 +630,7 @@ get_hash(ser)
 
 
 
-EXPORT void
+LOCAL void
 clean_up()
 {
 	uname(&un);
@@ -635,37 +640,17 @@ clean_up()
 			fclose(gpkt.p_iop);
 			gpkt.p_iop = NULL;
 		}
-		if (Xiop) {
-			fclose(Xiop);
-			Xiop = NULL;
+		if (gpkt.p_xiop) {
+			fclose(gpkt.p_xiop);
+			gpkt.p_xiop = NULL;
 			unlink(auxf(gpkt.p_file, 'x'));
 		}
 		if (pk2.p_iop) {
 			fclose(pk2.p_iop);
 			pk2.p_iop = NULL;
 		}
-		xrm();
+		xrm(&gpkt);
 		ffreeall();
 		unlockit(auxf(gpkt.p_file, 'z'), getpid(), uuname);
 	}
 }
-
-/*
- * Null routine to satisfy external reference from dodelt()
- */
-/*ARGSUSED*/
-void
-escdodelt(pkt)
-	struct packet *pkt;
-{
-}
-
-/*
- * NULL routine to satisfy external reference from dodelt()
- */
-/*ARGSUSED*/
-void
-fredck(pkt)
-	struct packet *pkt;
-{
-}
