@@ -1,8 +1,8 @@
-/* @(#)sccslog.c	1.34 11/09/03 Copyright 1997-2011 J. Schilling */
+/* @(#)sccslog.c	1.35 11/10/07 Copyright 1997-2011 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)sccslog.c	1.34 11/09/03 Copyright 1997-2011 J. Schilling";
+	"@(#)sccslog.c	1.35 11/10/07 Copyright 1997-2011 J. Schilling";
 #endif
 /*
  *	Copyright (c) 1997-2011 J. Schilling
@@ -40,13 +40,13 @@ struct xx {
 
 #define	PRINTED	0x01
 
-struct	xx	*list;
-int		listmax;
-int		listsize;
+LOCAL	struct	xx	*list;
+LOCAL	int		listmax;
+LOCAL	int		listsize;
 
-char		*Cwd;
-char		*SccsPath = "";
-BOOL		extended = FALSE;
+LOCAL	char		*Cwd;
+LOCAL	char		*SccsPath = "";
+LOCAL	BOOL		extended = FALSE;
 
 LOCAL	int	xxcmp		__PR((const void *vp1, const void *vp2));
 LOCAL	char *	mapuser		__PR((char *name));
