@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2011 J. Schilling
  *
- * @(#)val.c	1.30 11/09/16 J. Schilling
+ * @(#)val.c	1.35 11/10/13 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)val.c 1.30 11/09/16 J. Schilling"
+#pragma ident "@(#)val.c 1.35 11/10/13 J. Schilling"
 #endif
 /*
  * @(#)val.c 1.22 06/12/12
@@ -69,9 +69,6 @@
 #define	COPY	0
 #define	NOCOPY	1
 
-struct stat Statbuf;
-char	SccsError[MAXERRORLEN];
-
 static int	silent;		/* be silent, report only in exit code */
 static int	debug;		/* print debug messages */
 static int	ret_code;	/* prime return code from 'main' program */
@@ -223,10 +220,11 @@ char	*argv[];
 	register int	j;
 	register int	line_sw;
 	register char   *p;
+	register int	i;
 
 	int	num_files;
 
-	char	*filelist[50];
+	char	**filelist = NULL;
 	char	*savelinep;
 	int 	c;
 
@@ -343,9 +341,8 @@ char	*argv[];
 
 	for(j=1; j<argc; j++){
 		if(argv[j]) {
-			if (num_files >= 50)
-				fatal(gettext("too many files (va1)"));
-			filelist[num_files] = argv[j];
+			if (filelist == NULL)
+				filelist = &argv[j];
 		        num_files++;
 		}
 	}
@@ -373,16 +370,24 @@ char	*argv[];
 	/*
 	loop through 'validate' for each file on command line.
 	*/
-	for (j = 0; j < num_files; j++) {
+	for (i = 0, j = 0; j < num_files; j++) {
+		while (filelist[i+j] == NULL) {
+			i++;
+			if ((i+j+1) >= argc)
+				break;
+		}
+		if (filelist[i+j] == NULL)
+			break;
+
 		/*
 		read a file from 'filelist' and place into 'path'.
 		*/
-		if (size(filelist[j]) > FILESIZE) {
+		if (size(filelist[i+j]) > FILESIZE) {
 			extern char *Ffile;
-			Ffile = filelist[j];
+			Ffile = filelist[i+j];
 			fatal(gettext("too long (co7)"));
 		}
-		strlcpy(path, filelist[j], sizeof (path));
+		strlcpy(path, filelist[i+j], sizeof (path));
 		if (!inpstd) {
 			do_file(path, do_validate, 1, 1);
 			continue;
@@ -1238,7 +1243,7 @@ get_close()
 		fclose(pk2.p_iop);
 		pk2.p_iop = NULL;
 	}
-	xrm();
+	xrm(&gpkt);
 	ffreeall();
 }
 
@@ -1289,30 +1294,3 @@ get_hashtest(ser)
 	}
 	return (1);
 }
-
-
-/*
- * Null routine to satisfy external reference from dodelt()
- */
-/*ARGSUSED*/
-void
-escdodelt(pkt)
-	struct packet *pkt;
-{
-}
-
-/*
- * NULL routine to satisfy external reference from dodelt()
- */
-/*ARGSUSED*/
-void
-fredck(pkt)
-	struct packet *pkt;
-{
-}
-
-/* for fatal() */
-void
-clean_up()
-{
-}
