@@ -25,12 +25,12 @@
  * Use is subject to license terms.
  */
 /*
- * This file contains modifications Copyright 2006-2011 J. Schilling
+ * Copyright 2006-2011 J. Schilling
  *
- * @(#)dodelt.c	1.13 11/09/14 J. Schilling
+ * @(#)dodelt.c	1.17 11/10/13 J. Schilling
  */
 #if defined(sun)
-#pragma ident "@(#)dodelt.c 1.13 11/09/14 J. Schilling"
+#pragma ident "@(#)dodelt.c 1.17 11/10/13 J. Schilling"
 #endif
 /*
  * @(#)dodelt.c 1.8 06/12/12
@@ -50,11 +50,6 @@ static void	doixg	__PR((char *,struct ixg **));
 
 time_t	Timenow;
 
-char	Pgmr[LOGSIZE];	/* for rmdel & chghist (rmchg) */
-int	First_esc;
-int	First_cmt;
-int	CDid_mrs;		/* for chghist to check MRs */
-
 struct idel *
 dodelt(pkt,statp,sidp,type)
 register struct packet *pkt;
@@ -95,12 +90,12 @@ char type;
 		}
 		if (dt.d_type == 'D') {
 			if (sidp && eqsid(&dt.d_sid,sidp)) {
-				copy(dt.d_pgmr,Pgmr);	/* for rmchg */
+				copy(dt.d_pgmr, pkt->p_pgmr);	/* for rmchg */
 				zero((char *) sidp,sizeof(*sidp));
 				founddel = 1;
-				First_esc = 1;
-				First_cmt = 1;
-				CDid_mrs = 0;
+				pkt->p_first_esc = 1;
+				pkt->p_first_cmt = 1;
+				pkt->p_cdid_mrs = 0;
 				for (p = pkt->p_line; *p && *p != 'D'; p++)
 					;
 				if (*p) {
@@ -115,15 +110,13 @@ char type;
 					*p = type;
 				}
 				if (type == 0) {
-					extern	FILE	*Xiop;
-
 					/*
 					 * Go back before last stats line. The
 					 * value 21 is the lenght of any stats
 					 * line. Never change this length.
 					 */
 					pkt->p_nhash -= lhash;
-					fseek(Xiop, (off_t)-21, SEEK_CUR);
+					fseek(pkt->p_xiop, (off_t)-21, SEEK_CUR);
 					/*
 					 * Skip this delta table entry.
 					 */
@@ -138,7 +131,7 @@ char type;
 				}
 			}
 			else
-				First_esc = founddel = First_cmt = 0;
+				pkt->p_first_esc = pkt->p_first_cmt = founddel = 0;
 			pkt->p_maxr = max(pkt->p_maxr,dt.d_sid.s_rel);
 			rdp = &pkt->p_idel[dt.d_serial];
 			rdp->i_sid.s_rel = dt.d_sid.s_rel;
@@ -163,10 +156,10 @@ char type;
 				case MRNUM:
 					if (founddel)
 					{
-						escdodelt(pkt);
+						(*pkt->p_escdodelt)(pkt);
 						if(type == 'R' && HADZ && pkt->p_line[1] == MRNUM )
 						{
-							fredck(pkt);
+							(*pkt->p_fredck)(pkt);
 						}
 					}
 				continue;
